DenseRangeMapBuilder

Struct DenseRangeMapBuilder 

struct DenseRangeMapBuilder<K, V1, V2> {
    starts: Vec<K>,
    values1: Vec<Option<V1>>,
    values2: Option<Vec<Option<V2>>>,
    prev_end: Option<K>,
}
A helper type to create a DenseRangeMap from a sorted list of disjoint ranges.

Invariants

  • starts.len() == values.len()
  • starts is sorted and contains no duplicates.
  • starts is empty if and only if prev_end is None.

Semantics

If starts is empty, nothing has been added to this Builder.

Otherwise:

  • Every key such that K::min <= key < starts[0] maps to None.
  • Every key such that starts[idx] <= key < starts[idx+1] maps to values[idx], which may be Some or None.
  • Every key such that key <= starts.last() <= prev_end maps to values.last().
  • No mappings have been added for any range S..=E such that ’S > prev_end`.

Fields

starts: Vec<K>

A list of range starts so far.

values1: Vec<Option<V1>>

A list of values so far.

values2: Option<Vec<Option<V2>>>

A list of secondary values so far.

None if we’re ignoring secondary values.

prev_end: Option<K>

The last element of the most recently added range.

Implementations

Source

impl<K, V1, V2> DenseRangeMapBuilder<K, V1, V2>
where K: Clone + 'static + Eq + Ord + Successor, V1: Clone + 'static, V2: Clone + 'static,

fn new() -> Self

Construct a new empty builder.

fn push(&mut self, start: K, v1: Option<V1>, v2: Option<V2>)

Add a single entry to this builder.

fn build(self) -> DenseRangeMap<K, V1, V2>

Consume this builder and return a DenseRangeMap with the same values.

fn add_entry( &mut self, range: RangeInclusive<K>, value1: Option<V1>, value2: Option<V2>, ) -> Result<(), Error>

Add an entry to this DenseRangeMapBuilder.

Returns an error if range is not in strictly ascending order with respect to all previous ranges.