pub struct RawGeoipDbExport<'a> {
pub ipv4_starts: &'a [u32],
pub ipv4_ccs: &'a [Option<CountryCode>],
pub ipv4_asns: Option<&'a [Option<NonZeroU32>]>,
pub ipv6_starts: &'a [u128],
pub ipv6_ccs: &'a [Option<CountryCode>],
pub ipv6_asns: Option<&'a [Option<NonZeroU32>]>,
}Expand description
An export of a GeoIp database in a raw format suitable for embedding.
This format is deliberately undocumented, and not for other uses.
Fields
ipv4_starts: &'a [u32]ipv4_ccs: &'a [Option<CountryCode>]ipv4_asns: Option<&'a [Option<NonZeroU32>]>ipv6_starts: &'a [u128]ipv6_ccs: &'a [Option<CountryCode>]ipv6_asns: Option<&'a [Option<NonZeroU32>]>Implementations